#!/bin/sh
set -e

if [ -n "$1" ]; then
  lowercaseParam=$(echo $1 | tr '[:upper:]' '[:lower:]')
  if [[ $lowercaseParam =~ ^[[:digit:]a-f]{1,40}$ ]]; then
    startString=$lowercaseParam
  else
    echo "Error: Invalid parameter '$1'. (If a parameter is provided, it must be a hex string with no more than 40 characters.)"
    exit 1
  fi
else
  startString='0000000'
fi

lastCommitMessage=$(git --no-pager show -s --oneline --format="%B" head)
lastCommitDataWithNewline=$(git cat-file commit head | sed -E 's/\n$//')
lastCommitData=${lastCommitDataWithNewline::${#lastCommitDataWithNewline}-1}

currentNumber=0
currentHash=

TAB_CHARACTER=$(printf '\t')
until [[ $currentHash == $startString* ]]; do
  ((currentNumber++))
  currentAddition=$(echo "obase=2;$currentNumber" | bc | tr '0' ' ' | tr '1' "$TAB_CHARACTER")
  trimmedAddition=${currentAddition::${#currentAddition}-1}
  currentHash=$(echo -ne "commit $((${#lastCommitData} + ${#currentAddition} + 1))\0$lastCommitData$trimmedAddition" | shasum)
done
echo -ne "commit $((${#lastCommitData} + ${#currentAddition} + 1))\0$lastCommitData$trimmedAddition\n"

export GIT_COMMITTER_DATE=$(git --no-pager show -s --oneline --format="%cd" head)
git commit --amend --cleanup=verbatim -m "$lastCommitMessage$trimmedAddition"
